﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Collider),  typeof(CharacterController))]
public class Character : MonoBehaviour {
    public LayerMask deathMask;

    public Ray rayPrefab;
    public Texture2D rayTexture;
    protected Ray myRay;
    public Transform shootPoint;
    public Transform characterVisual;



    protected CharacterController c;

    public float gravity = 5;

    protected enum states { initial, playing, finished};
    protected states currentState = states.initial;

    protected GameManager manager;

    public string AimInput="Aim1", ShootInput="shoot", JumpInput="jump", HorizontalInput = "Horizontal", VerticalInput = "Vertical", FrequencyInput = "Frequency", AmplitudeInput="Amplitude";

    [Header("Ray growth values")]
    public float minRayLength = 2, maxRayLength = 60, rayLengthGrowthPerSecond = 5;
    protected float rayChargeAccumTime = 0;

    [Header("Sounds")]
    public AudioSource jumpEmitter, shootEmitter, chargeEmitter;
    public AudioClip jumpClip, shootClip, chargeClip;

    [Header("Charge")]
    public GameObject chargeRoot;
    public float chargeTime = 3, shootDuration=0.5f;

    [Header("ScreenShake")]
    protected CameraShaker shaker;
    public float shakeAmount = 1, shakeDecrease = 2, shakeForce = 1;

    // Use this for initialization
    void Start ()
    {
        shaker = FindObjectOfType<CameraShaker>();
        c = GetComponent<CharacterController>();
	}
    bool allowShoot = false;
    protected float timeSinceLastShoot = Mathf.Infinity;

    // Update is called once per frame

    protected bool shouldResetLength = false;
    void Update()
    {
        

        if (currentState == states.playing)
        {
            myRay.maxDrawDistance += Time.deltaTime * rayLengthGrowthPerSecond;
            myRay.maxDrawDistance = myRay.maxDrawDistance > maxRayLength ? maxRayLength : myRay.maxDrawDistance;


            timeSinceLastShoot += Time.deltaTime;

            allowShoot = false;
            if (!myRay.IsShooting)
            {
                
               
                movement();

                if (timeSinceLastShoot > chargeTime)
                {
                    chargeEmitter.Stop();
                    myRay.Show = true;

                    if (shouldResetLength)
                    {
                        myRay.maxDrawDistance = minRayLength;
                        shouldResetLength = false;
                    }
                        
                    chargeRoot.SetActive(false);
                    if (Input.GetButtonDown(ShootInput))
                    {
                        shaker.Shake(shakeAmount,shakeDecrease,shakeForce);
                        myRay.shoot();
                        timeSinceLastShoot = 0;
                        shootEmitter.PlayOneShot(shootClip);
                        chargeEmitter.PlayOneShot(chargeClip);
                        shouldResetLength = true;
                    }
                }
                else
                {
                    
                    myRay.Show = false;

                }

                
            }
            else
            {
                chargeRoot.SetActive(true);
            }
            //if (allowShoot && Input.GetAxis(ShootInput) > 0.5f)
            

            aim();
        }

    }

    public string myCollidersTag="player";
    public void StartPlaying(float rayDir, string colliderTag, GameManager manager)
    {
        this.manager = manager;
        if (myRay != null)
        {
            Destroy(myRay);
        }

        myRay = Instantiate(rayPrefab);
        myRay.GetComponent<LineRenderer>().material.mainTexture = rayTexture;
        myRay.initialPosition = shootPoint;
        myRay.rayDir = rayDir;
        myRay.colliderTag = colliderTag;
        myRay.maxDrawDistance = minRayLength;
        myRay.ShootTime = shootDuration;
        myCollidersTag = colliderTag;

        currentState = states.playing;
        verticalSpeed = 0;
    }
    
    public float moveSpeed = 10;

    public float jumpInitialSpeed = 40;

    protected float verticalSpeed = 0;
    public float gravityOnFall = 5;

    protected float verticalSpeedOnFloor = -5;


    public float frecuencyAimSpeed = 0.1f, amplitudeAimSpeed = 0.1f;
    protected void aim()
    {
        var freq = Input.GetAxis(FrequencyInput);

        myRay.Frequency += frecuencyAimSpeed * freq * Time.deltaTime;

        var amp = Input.GetAxis(AmplitudeInput);

        myRay.Amplitude += amplitudeAimSpeed * amp * Time.deltaTime;
        
    }

 

    protected void movement()
    {
        //var dir = controls.checkKey(InputBase.keys.left) && controls.checkKey(InputBase.keys.right) ? 0 : 
        //    controls.checkKey(InputBase.keys.left) ? -1 : 
        //    controls.checkKey(InputBase.keys.right) ? 1 : 0;
        var dir = Input.GetAxis(HorizontalInput);

        var jump = Input.GetButton(JumpInput);
        var jumpDown = Input.GetButtonDown(JumpInput);

        var aiming = Input.GetAxis(AimInput) > 0.5f;

        //var jump = controls.checkKey(InputBase.keys.jump);
        //var jumpDown = controls.checkKeyDown(InputBase.keys.jump);

        myRay.Show = true;

        if (dir != 0 && !aiming)
        {
            var scale = characterVisual.localScale;
            scale.x = dir > 0 ? 1 : -1;
            characterVisual.localScale = scale;
            myRay.rayDir = scale.x;
            //myRay.Show = false;
        }
        //else 
        //{
        //    myRay.Show = !jump ;
        //}

        if(dir==0 && !jump && !jumpDown && c.isGrounded)
        {
            allowShoot = true;
        }
       
       

        if(jumpDown && c.isGrounded)
        {
            jumpEmitter.PlayOneShot(jumpClip);
            verticalSpeed = jumpInitialSpeed;
        }

    
        var move = new Vector3(dir, verticalSpeed, 0) * moveSpeed * Time.deltaTime;
        verticalSpeed -= (jump ? gravity : gravityOnFall)* Time.deltaTime;

       if(c.isGrounded && !jumpDown)
        {
            verticalSpeed = -1;
        }

        c.Move(move);
        
    }

    void OnTriggerEnter(Collider other)
    {
        int objLayerMask = (1 << other.gameObject.layer);
        if (other.gameObject!=gameObject && currentState==states.playing &&  ((deathMask.value & objLayerMask) > 0)) { // Extra round brackets required!
            if(other.gameObject.tag != myCollidersTag)
            {
                manager.playerLost(this);
            }
        }
            
    }

    public void setEnd()
    {
        if (myRay != null)
        {
            Destroy(myRay.gameObject);
        }
        currentState = states.finished;
    }
}
