﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[System.Serializable]
public class CharacterInfo
{
    public Sprite headOnSelectionWindow;
    public Character prefab;
}

public class GameManager : MonoBehaviour {
  

    [Header("Intro")]
    public GameObject introScreenContent;
    public Transform introScreen;
    public float introScreenSecondsDown = 4, introScreenSecondsWaiting=3, introScreenHeight = 40;

    [Header("Player Selection")]
    public GameObject characterSelectionContent;
    public CharacterInfo[] players;
    public SpriteRenderer p1Head, p2Head;
    protected int currentP1Head=0, currentP2Head=1;
    public float timeBetweenChanges = 0.5f;
    public GameObject p1Selected, p2Selected;


    [Header("Characters ingame")]
    public Transform player1InitialPos, player2InitialPos;
    protected Character player1, player2;

    [Header("Character controls")]
    protected string horizontalAlias = "Horizontal", verticalAlias = "Vertical", frecuencyAlias = "Frequency", amplitudeAlias = "Amplitude", fireAlias = "Fire", jumpAlias = "Jump", AimAlias = "Aim";

    protected bool matchDecided = false;

    [Header("End screen")]
    public Transform endSceneRoot;
    public SpriteRenderer endHelmetWinner, endHelmetLoser;
    public float endScreenTimeSeconds = 1;

    [Header("Sounds")]
    public AudioSource end;
    public AudioClip endClip;
    public UnityEngine.Audio.AudioMixerSnapshot menuSnapshot, battleSnapshot, winSnapshot;

    [Header("ScreenShake")]
    public CameraShaker shaker;
    public float shakeAmount = 1, shakeDecrease = 1, shakeForce = 1;

    // Use this for initialization
    void Start () {
         StartCoroutine(GameLoopCoroutine());
	}



   protected void setupInitialHeads()
    {
        p1Head.sprite = players[currentP1Head].headOnSelectionWindow;
        p2Head.sprite = players[currentP2Head].headOnSelectionWindow;
        p1Selected.SetActive(false);
        p2Selected.SetActive(false);

    }

    protected void setPlayerControls(Character c, int player)
    {
        c.AimInput = AimAlias + player;
        c.HorizontalInput = horizontalAlias + player;
        c.VerticalInput = verticalAlias + player;
        c.ShootInput = fireAlias + player;
        c.JumpInput = jumpAlias + player;
        c.AmplitudeInput = amplitudeAlias + player;
        c.FrequencyInput = frecuencyAlias + player;
        
        
    }


    protected IEnumerator GameLoopCoroutine()
    {
        yield return null;
        menuSnapshot.TransitionTo(1);

        //get intro screen down
        introScreenContent.SetActive(true);
        characterSelectionContent.SetActive(false);

        endSceneRoot.gameObject.SetActive(false);

        var targetIntroFinish = introScreen.transform.position;
        var targetIntroStart = targetIntroFinish + Vector3.up * introScreenHeight;

        introScreen.transform.position = targetIntroStart;

        var accumTime = 0.0f;
        while (accumTime < 1)
        {
            accumTime += Time.deltaTime/introScreenSecondsDown;
            introScreen.transform.position = Vector3.Lerp(targetIntroStart, targetIntroFinish, accumTime);
            //TODO speedup if press
            yield return null;
        }
        //play intro
        yield return new WaitForSeconds(introScreenSecondsWaiting);

        introScreenContent.SetActive(false);
    

        while (true)
        {

            endSceneRoot.gameObject.SetActive(false);
            
            characterSelectionContent.SetActive(true);
            //setup player selection
            setupInitialHeads();

            //wait for players
            bool p1Accepted = false, p2Accepted = false;
            float elapsedSinceP1Pressed = Mathf.Infinity, elapsedSinceP2Pressed=Mathf.Infinity;
            while (!(p1Accepted && p2Accepted))
            {
                var h1 = Input.GetAxis("Horizontal1");
                var h2 = Input.GetAxis("Horizontal2");

                elapsedSinceP1Pressed += Time.deltaTime;
                elapsedSinceP2Pressed += Time.deltaTime;

                if (!p1Accepted && (h1 > 0.5f && elapsedSinceP1Pressed>timeBetweenChanges || h1 < -0.5f && elapsedSinceP1Pressed > timeBetweenChanges))
                {
                    elapsedSinceP1Pressed = 0;
                    currentP1Head = currentP1Head + 1 >= players.Length ? 0 : currentP1Head + 1;
                    p1Head.sprite = players[currentP1Head].headOnSelectionWindow;
                }

                if (!p2Accepted && (h2 > 0.5f && elapsedSinceP2Pressed > timeBetweenChanges || h2 < -0.5f && elapsedSinceP2Pressed > timeBetweenChanges))
                {
                    elapsedSinceP2Pressed = 0;
                    currentP2Head = currentP2Head + 1 >= players.Length ? 0 : currentP2Head + 1;
                    p2Head.sprite = players[currentP2Head].headOnSelectionWindow;
                }

                if (Input.GetButtonDown("Jump1"))
                {
                    p1Accepted = true;
                }
                if (Input.GetButtonDown("Jump2"))
                {
                    p2Accepted = true;
                }
                if (Input.GetButtonDown("Start") || Input.GetKeyDown(KeyCode.Return))
                {
                    p1Accepted = true;
                    p2Accepted = true;
                }

                p1Selected.SetActive(p1Accepted);
                p2Selected.SetActive(p2Accepted);

                yield return null;

            }


            //set up players 
            if (player1 != null)
            {
                Destroy(player1);
            }
            if(player2 != null)
            {
                Destroy(player2);
            }

            player1 = Instantiate(players[currentP1Head].prefab);
            setPlayerControls(player1, 1);
            player1.transform.position = player1InitialPos.position;

            player2 = Instantiate(players[currentP2Head].prefab);
            setPlayerControls(player2, 2);
            player2.transform.position = player2InitialPos.position;

            var scale = player2.characterVisual.localScale;
            scale.x = -1;
            player2.characterVisual.localScale = scale;
       

            player1.transform.position = player1InitialPos.position;
            player2.transform.position = player2InitialPos.position;


            battleSnapshot.TransitionTo(1);

            //get player selection up

            accumTime = 0.0f;
            while (accumTime < 1)
            {
                accumTime += Time.deltaTime / introScreenSecondsDown;
                introScreen.transform.position = Vector3.Lerp(targetIntroFinish, targetIntroStart, accumTime);
                //TODO speedup if press
                yield return null;
            }

            characterSelectionContent.SetActive(false);
            //countdown
            matchDecided = false;
            //enable players control
            player1.StartPlaying(1,"Player1",this);
            player2.StartPlaying(-1,"Player2",this);

            //wait for winner
            while (!matchDecided)
            {
                yield return null;
            }

            shaker.Shake(shakeAmount, shakeDecrease, shakeForce);

            winSnapshot.TransitionTo(1);
            end.PlayOneShot(endClip);

            //setup winner screen
            characterSelectionContent.SetActive(false);
            endSceneRoot.gameObject.SetActive(true);
            endHelmetWinner.sprite = players[winner].headOnSelectionWindow;
            endHelmetLoser.sprite = players[loser].headOnSelectionWindow;

            //get winner screen down
            accumTime = 0.0f;
            while (accumTime < 1)
            {
                accumTime += Time.deltaTime / endScreenTimeSeconds;
                introScreen.transform.position = Vector3.Lerp(targetIntroStart,targetIntroFinish , accumTime);
                //TODO speedup if press
                yield return null;
            }

            Destroy(player1.gameObject);
            Destroy(player2.gameObject);
            Destroy(player1.gameObject);
            Destroy(player2.gameObject);

            //wait for restart
            while (!Input.GetButtonDown("Start") && !Input.GetKeyDown(KeyCode.Return))
            {
                yield return null;
            }

            menuSnapshot.TransitionTo(1);
            endSceneRoot.gameObject.SetActive(false);
            characterSelectionContent.SetActive(true);

            yield return null;
        }

    }

    protected int winner, loser;
    public void playerLost(Character c)
    {
        winner = c==player2 ? currentP1Head : currentP2Head;
        loser = c==player2 ? currentP2Head : currentP1Head;

        player1.setEnd();
        player2.setEnd();
        matchDecided = true;
    }

	// Update is called once per frame
	void Update () {
		
	}
}
