﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(LineRenderer))]
public class Ray : MonoBehaviour {
    public float xDistanceBetweenControlPoints = 1;
    public float maxDrawDistance=5;

    [Range(0.0f, 1.0f)]
    public float frequency = 1;
    [Range(0.0f, 15.0f)]
    public float amplitude = 1;

    public float lineWithOnAim = 0.5f, lineWidthOnShoot = 1.0f;

    protected LineRenderer line;

    public Transform initialPosition;

    public float alphaOnShoot = 1.0f, alphaOnAim = 0.3f;
    protected bool shooting = false;

    protected Transform collidersRoot;

    public float RayZOffset = 0;

    public float rayDir = 1;

    public string colliderTag;

    public bool IsShooting
    {
        get { return shooting; }
    }

    public float Frequency
    {
        get
        {
            return frequency;
        }
        set
        {
            if (value > 1) { frequency = 1; }
            else if (value < 0.1f) { frequency = 0.1f; }
            else { frequency = value; }
        }
    }

    public float Amplitude
    {
        get
        {
            return amplitude;
        }
        set
        {
            if (value > 10) { amplitude = 10; }
            else if (value < 0.1f) { amplitude = 0.1f; }
            else { amplitude = value; }
        }
    }



    // Use this for initialization
    void Start () {
        line = GetComponent<LineRenderer>();
        collidersRoot = new GameObject().transform;
        collidersRoot.parent = transform;
        collidersRoot.localPosition = Vector3.zero;
        collidersRoot.localRotation = Quaternion.identity;

        DrawLine();
	}
	
	// Update is called once per frame
	void Update () {
        if (!shooting)
        {
            DrawLine();
        }
        

    }

    public void shoot()
    {
        if (!shooting)
        {
            StartCoroutine(ShootCoroutine());
        }
    }
    protected float shootTime = 2;
    public float ShootTime
    {
        set
        {
            shootTime = value;
        }
    }
    protected IEnumerator ShootCoroutine()
    {
        shooting = true;

        line.widthMultiplier = lineWidthOnShoot;
        setLineAlpha(alphaOnShoot);
        var positions = new Vector3[line.numPositions];

        line.GetPositions(positions);

        //remove old colliders (if any)
        for (int i = 0; i < collidersRoot.childCount; i++)
        {
            Destroy(collidersRoot.GetChild(i).gameObject);
        }

        //make new colliders
        setColliders(positions);

        yield return new WaitForSeconds(shootTime);

        setLineAlpha(draw ? alphaOnAim : 0);

        //remove old colliders
        for (int i = 0; i < collidersRoot.childCount; i++)
        {
            Destroy(collidersRoot.GetChild(i).gameObject);
        }

        shooting = false;

        line.widthMultiplier = lineWithOnAim;
    }

    protected void setLineAlpha(float alpha)
    {
        var keys = line.colorGradient.alphaKeys;
        for(int i = 0; i < keys.Length - 1; i++)
        {
            keys[i].alpha = alpha;
        }
        //line.colorGradient.SetKeys(line.colorGradient.colorKeys, keys);

        line.colorGradient = new Gradient { colorKeys = line.colorGradient.colorKeys, alphaKeys = keys, mode = GradientMode.Blend };
        line.widthMultiplier = shooting ? 1 : 0.4f;
    }

    protected bool draw = true;
    public bool Show
    {
        get { return draw; }
        set { draw = value; }
    }

    protected void setColliders(Vector3[] points)
    {
        //make new colliders
        var lastPoint = points[0];
        for(int i = 1; i < points.Length; i++)
        {
            var current = points[i];

            var direction = (current - lastPoint).normalized;
            
            var newCol = new GameObject().AddComponent<BoxCollider>();
            newCol.transform.position = lastPoint + direction*Vector3.Distance(current,lastPoint)/2;
            newCol.transform.parent = collidersRoot;
            newCol.gameObject.layer = gameObject.layer;
            newCol.gameObject.tag = colliderTag;
            

            newCol.transform.parent = collidersRoot;

            newCol.transform.rotation = Quaternion.LookRotation(direction);

            var  size = newCol.size;
            size.z = Mathf.Abs(Vector3.Distance(current, lastPoint));
            size.x = 0.3f;
            size.y = 0.3f;

            newCol.size = size;
            lastPoint = current;
        }
    }

    public void DrawLine()
    {
        var initial = initialPosition.position;

        var distanceDrawn = 0.0f;

        line.widthMultiplier = lineWithOnAim;

        setLineAlpha(shooting ? alphaOnShoot : (draw ? alphaOnAim : 0));

        var points = new List<Vector3>();

        var realDistance = rayDir * xDistanceBetweenControlPoints / amplitude;

        points.Add(initial);

        while (distanceDrawn < maxDrawDistance)
        {
            var next = Mathf.Sin(points.Count * realDistance * frequency) *amplitude;

            var point = new Vector3(realDistance * points.Count, next, RayZOffset) + initial;
            
            distanceDrawn += Vector3.Distance(point, points[points.Count - 1]);
            
            points.Add(point);
        }
        line.numPositions = points.Count;
        line.SetPositions(points.ToArray());

     
       
    }
       
    }
